/**
 * Please add your custom config
 *
 * @version 4
 */
import Axios from "axios";
import { StatusCodes } from "hitobit-constants";
import { getDeviceId, URLManager } from "hitobit-modules";
import i18n from "i18next";
//@ts-ignore
import qs from "qs";
import { axiosMiddle, removeUser } from "../context/userManager";
let axiosInstance;
function getAxiosInstance(security) {
    if (!axiosInstance) {
        const baseConfig = {
            headers: {
                "Content-Encoding": "UTF-8",
                Accept: "application/json",
                "Content-Type": "application/json",
                clientId: __CLIENT_ID__,
                deviceId: getDeviceId(),
            },
            timeout: 200 * 1000,
            paramsSerializer: (param) => qs.stringify(param, { indices: false }),
        };
        axiosInstance = Axios.create(baseConfig);
        if (typeof window !== "undefined") {
            axiosMiddle(axiosInstance, (config) => config.data?.grantType === "RefreshToken");
        }
        // Response interceptor
        axiosInstance.interceptors.response.use((async (response) => {
            // Any status code that lie within the range of 2xx cause this function to trigger
            // Do something with response data
            /**
             * Example on response manipulation
             *
             * @example
             *   const swaggerResponse: SwaggerResponse = {
             *     ...response,
             *   };
             *   return swaggerResponse;
             */
            return response.data;
        }), async (error) => {
            // Any status codes that falls outside the range of 2xx cause this function to trigger
            // Do something with response error
            if (error.response) {
                if (error.response.status === StatusCodes.UserTrustedDevice) {
                    await removeUser();
                }
                const errors = error.response?.data?.errors || error.response?.data?.value?.errors;
                return Promise.reject(new RequestError({
                    message: error.response?.data?.message ||
                        error.response?.data?.msg ||
                        errors?.[0]?.message,
                    status: error.response.status || errors?.[0]?.statusCode,
                    response: error.response,
                    errors: errors,
                }));
            }
            if (error.isAxiosError) {
                return Promise.reject(new RequestError({
                    message: "noInternetConnection",
                }));
            }
            return Promise.reject(error);
        });
        axiosInstance.interceptors.request.use((request) => {
            request.headers = {
                ...request.headers,
                "Accept-Language": i18n.language,
            };
            return request;
        });
    }
    axiosInstance.defaults.baseURL = URLManager.baseUrl;
    return axiosInstance;
}
class RequestError extends Error {
    constructor({ message, status, response, errors }) {
        super(message);
        this.isApiException = true;
        this.message = message;
        this.status = status;
        this.response = response;
        this.errors = errors;
    }
    static isRequestError(error) {
        return error?.isApiException;
    }
    /** @deprecated Use isRequestError instead */
    static isApiException(error) {
        return error?.isApiException;
    }
}
export { getAxiosInstance, RequestError };
